#!/usr/bin/env python

import os
from argparse import ArgumentParser

from dtags.help import HelpFormatter
from dtags.colors import PINK, CYAN, YELLOW, CLEAR
from dtags.config import load_tags, save_tags
from dtags.utils import expand_path

cmd_description = """
dtags - untag directories

e.g. the command {y}untag ~/foo ~/bar @a @b ~/baz @c @d{x}:

    removes from directory {c}~/foo{x} tags {p}@a @b{x}
    removes from directory {c}~/bar{x} tags {p}@a @b{x}
    removes from directory {c}~/baz{x} tags {p}@c @d{x}

""".format(p=PINK, c=CYAN, y=YELLOW, x=CLEAR)

msg = "Removed tag {p}{{}}{x} from {c}{{}}{x}".format(p=PINK, c=CYAN, x=CLEAR)


def main():
    tag_to_paths = load_tags()
    parser = ArgumentParser(
        prog="untag",
        usage="untag [[paths] [tags]...]",
        description=cmd_description,
        formatter_class=HelpFormatter
    )
    parser.add_argument(
        "arguments",
        type=str,
        nargs='+',
        metavar='[paths] [tags]',
        help="directory paths and tag names"
    )
    parsed = parser.parse_args()

    # Tracking variables and collectors
    updates = []
    arg_index = 0
    parsing_paths = True
    tags_collected = set()
    paths_collected = set()

    # Iterate through the arguments and pair up tags with paths
    while arg_index < len(parsed.arguments):
        arg = parsed.arguments[arg_index]
        if parsing_paths and arg.startswith('@'):
            if len(paths_collected) == 0:
                parser.error("excepting paths before {}".format(arg))
            parsing_paths = False
        elif parsing_paths and not arg.startswith('@'):
            paths_collected.add(arg)
            arg_index += 1
        elif not parsing_paths and arg.startswith('@'):
            tags_collected.add(arg)
            arg_index += 1
        else:
            updates.append((tags_collected, paths_collected))
            tags_collected, paths_collected = set(), set()
            parsing_paths = True
    if parsing_paths:
        parser.error("expecting a tag name")
    updates.append((tags_collected, paths_collected))

    # Apply updates and message
    messages = set()
    for tags, paths in updates:
        for tag in tags:
            if tag not in tag_to_paths:
                continue
            for path in paths:
                if not os.path.isdir(path):
                    continue
                full_path = expand_path(path)
                if full_path in tag_to_paths[tag]:
                    tag_to_paths[tag].pop(full_path)
                    messages.add(msg.format(tag, full_path))
            if len(tag_to_paths[tag]) == 0:
                # Remove the tag completely if it has no paths
                tag_to_paths.pop(tag)
    save_tags(tag_to_paths)
    if messages:
        print("\n".join(messages))



#!/usr/bin/env python

import os
import sys
import json

from collections import defaultdict
from argparse import ArgumentParser

from dtags.colors import PINK, CYAN, YELLOW, CLEAR
from dtags.completion import ChoicesCompleter, autocomplete
from dtags.config import load_tags
from dtags.help import HelpFormatter
from dtags.utils import expand_path

cmd_description = """
dtags - display tags and tagged directories

e.g. the command {y}tags @a @b ~/foo ~/bar @c{x}:

    displays tags {p}@a @b @c{x}
    displays all tags with directory {c}~/foo{x}
    displays all tags with directory {c}~/bar{x}

""".format(p=PINK, c=CYAN, y=YELLOW, x=CLEAR)


def main():
    tag_to_paths = load_tags()
    path_to_tags = defaultdict(set)
    for tag, paths in tag_to_paths.items():
        for path in paths.keys():
            path_to_tags[path].add(tag)

    parser = ArgumentParser(
        prog="tags",
        description=cmd_description,
        usage="tags [options] [paths] [tags]",
        formatter_class=HelpFormatter
    )
    parser.add_argument(
        "-e", "--expand",
        action="store_true",
        help="Expand the directory paths"
    )
    parser.add_argument(
        "-r", "--reverse",
        help="display the reverse mapping",
        action="store_true"
    )
    parser.add_argument(
        "-j", "--json",
        help="display the raw JSON",
        action="store_true"
    )
    parser.add_argument(
        "search_terms",
        type=str,
        nargs='*',
        metavar="[paths] [tags]",
        help="tag and directory paths to filter by"
    ).completer = ChoicesCompleter(tag_to_paths.keys())
    autocomplete(parser)
    parsed = parser.parse_args()

    if len(tag_to_paths) == 0:
        print("No tags found! You can add them by running the 'tag' command.")
        sys.exit(0)

    # Filter by any given tags and paths
    # TODO optimize here if possible
    if not parsed.search_terms:
        if parsed.expand:
            filtered = {t: ps.keys() for t, ps in tag_to_paths.items()}
        else:
            filtered = {t: ps.values() for t, ps in tag_to_paths.items()}
    else:
        filtered = {}
        for term in parsed.search_terms:
            if term in tag_to_paths:
                if parsed.expand:
                    filtered[term] = tag_to_paths[term].keys()
                else:
                    filtered[term] = tag_to_paths[term].values()
            elif os.path.isdir(term):
                term = expand_path(term)
                if term in path_to_tags:
                    for tag in path_to_tags[term]:
                        if parsed.expand:
                            filtered[tag] = tag_to_paths[tag].keys()
                        else:
                            filtered[tag] = tag_to_paths[tag].values()

    if parsed.json:
        formatted = {tag: sorted(paths) for tag, paths in filtered.items()}
        print(json.dumps(formatted, sort_keys=True, indent=4))
    elif parsed.reverse:
        reverse = defaultdict(set)
        for tag, paths in filtered.items():
            for path in paths:
                reverse[path].add(tag)
        for path, tags in reverse.items():
            print("{}{}{}".format(CYAN, path, CLEAR))
            print("{}{}{}\n".format(PINK, " ".join(sorted(tags)), CLEAR))
    else:
        for tag, paths in sorted(filtered.items()):
            print("{}{}{}".format(PINK, tag, CLEAR))
            print("{}{}{}\n".format(CYAN, "\n".join(sorted(paths)), CLEAR))



from setuptools import setup, find_packages

setup(
    name="dtags",
    version="1.0.4",
    description="Directory tags for lazy programmers",
    author="Joohwan Oh",
    author_email="joohwan.oh@outlook.com",
    url="https://github.com/joowani/dtags",
    download_url="https://github.com/joowani/dtags/tarball/1.0.4",
    packages=find_packages(),
    install_requires=["argcomplete"],
    entry_points={
        'console_scripts': [
            'run = dtags.commands.run:main',
            'tag = dtags.commands.tag:main',
            'tags = dtags.commands.tags:main',
            'untag = dtags.commands.untag:main',
        ],
    },
)



import os
import json

from dtags.utils import halt, expand_path

TAGS_FILE = expand_path('~/.dtags')


def load_tags():
    """Load the tags from disk.

    Return a dictionary which maps tag names to sub-dictionaries, which in
    turn maps fully expanded paths to unexpanded paths like this:

    {
        "@app" : {
            "~/app/backend": "/home/user/app/backend",
            "~/app/frontend": "/home/user/app/frontend",
        },
        "@frontend": {
            "~/app/frontend": "/home/user/app/frontend",
        },
        "@backend": {
            "~/app/backend": "/home/user/app/backend",
        }
        ...
    }

    :return: the tags loaded from the disk
    """
    if not os.path.exists(TAGS_FILE):
        try:
            with open(TAGS_FILE, "w") as config_file:
                json.dump({}, config_file)
            return {}
        except (IOError, OSError):
            halt("Failed to initialize {}".format(TAGS_FILE))
    else:
        try:
            with open(TAGS_FILE, "r") as config_file:
                json_str = config_file.read().strip()
                if not json_str:
                    return {}
                tag_data = json.loads(json_str)
                if not tag_data:
                    return {}
                else:
                    return {
                        tag: {expand_path(path): path for path in paths}
                        for tag, paths in tag_data.items()
                    }
        except (ValueError, IOError, OSError):
            # TODO better error handling and messaging here
            halt("Failed to load {}".format(TAGS_FILE))


def save_tags(tags):
    """Save the tags to disk.

    Convert the incoming tags dictionary to JSON and dump the content to the
    file. For portability, only the unexpanded paths are saved.

    :param tags: tags to save to disk
    """
    try:
        with open(TAGS_FILE, "w") as config_file:
            json.dump(
                {tag: sorted(paths.values()) for tag, paths in tags.items()},
                config_file,
                indent=4,
                sort_keys=True
            )
    except IOError:
        # TODO better error handling and messaging here
        halt("Failed to write to {}: {}".format(TAGS_FILE))
